import { Prop, Provide, Emit, Model } from 'vue-property-decorator';
import { Subject, Subscription } from 'rxjs';
import { Watch, GridControlBase } from '@/studio-core';
import HREmployeeService from '@/service/hremployee/hremployee-service';
import MainService from './main-grid-service';
import HREmployeeUIService from '@/uiservice/hremployee/hremployee-ui-service';
import { FormItemModel } from '@/model/form-detail';


/**
 * grid部件基类
 *
 * @export
 * @class GridControlBase
 * @extends {MainGridBase}
 */
export class MainGridBase extends GridControlBase {

    /**
     * 获取部件类型
     *
     * @protected
     * @type {string}
     * @memberof MainGridBase
     */
    protected controlType: string = 'GRID';

    /**
     * 建构部件服务对象
     *
     * @type {MainService}
     * @memberof MainGridBase
     */
    public service: MainService = new MainService({ $store: this.$store });

    /**
     * 实体服务对象
     *
     * @type {HREmployeeService}
     * @memberof MainGridBase
     */
    public appEntityService: HREmployeeService = new HREmployeeService({ $store: this.$store });

    /**
     * 应用实体名称
     *
     * @protected
     * @type {string}
     * @memberof MainGridBase
     */
    protected appDeName: string = 'hremployee';

    /**
     * 应用实体中文名称
     *
     * @protected
     * @type {string}
     * @memberof MainGridBase
     */
    protected appDeLogicName: string = '员工';

    /**
     * 界面UI服务对象
     *
     * @type {HREmployeeUIService}
     * @memberof MainBase
     */  
    public appUIService:HREmployeeUIService = new HREmployeeUIService(this.$store);

    /**
     * 界面行为模型
     *
     * @type {*}
     * @memberof MainBase
     */  
    public ActionModel: any = {
    };

    /**
     * 本地缓存标识
     *
     * @protected
     * @type {string}
     * @memberof MainBase
     */
    protected localStorageTag: string = 'hremployee_main_grid';

    /**
     * 所有列成员
     *
     * @type {any[]}
     * @memberof MainGridBase
     */
    public allColumns: any[] = [
        {
            name: 'employeecode',
            label: '员工编号',
            langtag: 'entities.hremployee.main_grid.columns.employeecode',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'employeename',
            label: '员工姓名',
            langtag: 'entities.hremployee.main_grid.columns.employeename',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'organizationname',
            label: '单位',
            langtag: 'entities.hremployee.main_grid.columns.organizationname',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'certnum',
            label: '证件号码',
            langtag: 'entities.hremployee.main_grid.columns.certnum',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'birthday',
            label: '出生日期',
            langtag: 'entities.hremployee.main_grid.columns.birthday',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'age',
            label: '年龄',
            langtag: 'entities.hremployee.main_grid.columns.age',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'mobile',
            label: '移动电话',
            langtag: 'entities.hremployee.main_grid.columns.mobile',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'email',
            label: '电子邮箱',
            langtag: 'entities.hremployee.main_grid.columns.email',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
        {
            name: 'empstate',
            label: '员工状态',
            langtag: 'entities.hremployee.main_grid.columns.empstate',
            show: true,
            util: 'PX',
            isEnableRowEdit: false,
        },
    ]

    /**
     * 获取表格行模型
     *
     * @type {*}
     * @memberof MainGridBase
     */
    public getGridRowModel(){
        return {
          srfkey: new FormItemModel(),
        }
    }

    /**
     * 属性值规则
     *
     * @type {*}
     * @memberof MainGridBase
     */
    public rules: any = {
        srfkey: [
            { required: false, validator: (rule:any, value:any, callback:any) => { return (rule.required && (value === null || value === undefined || value === "")) ? false : true;}, message: '员工标识 值不能为空', trigger: 'change' },
            { required: false, validator: (rule:any, value:any, callback:any) => { return (rule.required && (value === null || value === undefined || value === "")) ? false : true;}, message: '员工标识 值不能为空', trigger: 'blur' },
        ],
    }

    /**
     * 获取对应列class
     *
     * @type {*}
     * @memberof MainBase
     */
    public hasRowEdit: any = {
        'employeecode':false,
        'employeename':false,
        'organizationname':false,
        'certnum':false,
        'birthday':false,
        'age':false,
        'mobile':false,
        'email':false,
        'empstate':false,
    };

    /**
     * 获取对应列class
     *
     * @param {*} $args row 行数据，column 列数据，rowIndex 行索引，列索引
     * @returns {void}
     * @memberof MainBase
     */
    public getCellClassName(args: {row: any, column: any, rowIndex: number, columnIndex: number}): any {
        return ( this.hasRowEdit[args.column.property] && this.actualIsOpenEdit ) ? "edit-cell" : "info-cell";
    }


    /**
     * 导出数据格式化
     *
     * @param {*} filterVal
     * @param {*} jsonData
     * @param {any[]} [codelistColumns=[]]
     * @returns {Promise<any>}
     * @memberof MainGridBase
     */
    public async formatExcelData(filterVal: any, jsonData: any, codelistColumns?: any[]): Promise<any> {
        return super.formatExcelData(filterVal, jsonData, [
            {
                name: 'empstate',
                srfkey: 'CL_HR_0016',
                codelistType : 'DYNAMIC',
                renderMode: 'other',
                textSeparator: '、',
                valueSeparator: ',',
            },
        ]);
    }

}